from http.server import HTTPServer, BaseHTTPRequestHandler
from socket import gethostbyname, gethostname
import json
from eqsmart.main.provider import not_c_key


def preset_get_params(request):
    """
    GET 参数列表格式化
    :param request: GET请求链接
    :return: service_path 服务地址, params 参数列表
    """
    service_path = []
    params = {}
    try:
        path_params = request.split('?')
        service_path = path_params[0][1:].split('/')
        params_item = path_params[1].split('&')
        for item in params_item:
            key_value = item.split('=')
            params[key_value[0]] = key_value[1]
    except Exception as e:
        print('[eqsmart] [GET 参数读取失败]', e)
    return service_path, params


def func_exec(service_list, service_path, params):
    """
    函数执行
    :param service_list: 服务列表
    :param service_path: 服务方法的路径
    :param params: 服务方法入参
    :return: 执行结果
    """
    print(service_list)
    try:
        for i in service_path[:-1]:
            service_list = service_list[i]
        method = getattr(service_list['func'], service_path[-1])
        response = method(**params)
    except Exception as e:
        print('[eqsmart] [函数执行失败]', e)
        response = 'Function execution failed! Details: ' + str(e)
    return response


class MyHttpServer(BaseHTTPRequestHandler):
    my_server_list = {}
    server_version = "Apache"
    sys_version = 'Python/3'

    def do_GET(self):
        """
        处理GET请求
        :return: http response
        """
        path = self.path
        service_path, params = preset_get_params(path)
        res = func_exec(self.my_server_list, service_path, params)
        self.send_response(200)
        self.send_header("Content-type", "application/json")
        self.send_header("test1", "This is test!")
        self.end_headers()
        self.wfile.write(json.dumps(res).encode())

    def do_POST(self):
        """
        处理POST请求
        :return: http response
        """
        path = self.path
        data = self.rfile.read(int(self.headers['content-length']))
        data = json.loads(data)
        service_path = path[1:].split('/')
        res = func_exec(self.my_server_list, service_path, data)
        self.send_response(200)
        self.send_header("Content-type", "application/json")
        self.send_header("test", "This is post!")
        self.end_headers()
        self.wfile.write(json.dumps(res).encode())


def http_server(server_conf, server_list):
    """
    实例化http服务器对象
    :param server_conf: http服务器配置
    :param server_list: http服务列表
    :return: http服务器对象
    """
    host = gethostbyname(gethostname()) if not_c_key(server_conf, 'HOST') else server_conf['HOST']
    mhs = MyHttpServer
    mhs.my_server_list = server_list
    print(f"[eqsmart] HTTP Server starting on {host}:{server_conf['PORT']}")
    server = HTTPServer((host, server_conf['PORT']), mhs)
    return server


def main():
    hs = http_server({'PORT': 9701}, {'user_service': TestCommon()})
    hs.serve_forever()


class TestCommon:
    name = 'eqsmart'
    age = '1'

    def add_user(self, name, age):
        self.name = name
        self.age = age
        return 'hello' + name


if __name__ == '__main__':
    main()



import socket
from json import dumps as json_dumps
import traceback


class Consumer:
    def __init__(self, provider_conf):
        """
        对象初始化
        :param provider_conf: 服务提供着地址及其他配置
        """
        self.server_conf = provider_conf

    def func_call_int(self, send_data):
        """
        服务提供者注册
        :return: None
        """
        try:
            connect_provider = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        except Exception as e:
            print('[eqsmart] [consumer] [Socket创建] [ERROR]:' + str(e))
            print(traceback.format_exc())
            return json_dumps({'code': 'remote_call_error', 'message': str(e)})

        '''连接服务提供者'''
        try:
            connect_provider.connect((self.server_conf['IP'], self.server_conf['PORT']))
            connect_provider.setblocking(True)  # 设置阻塞模式，等待Provider调用的返回
            print(f"[eqsmart] [consumer] connect provider success {self.server_conf['IP']}:{self.server_conf['PORT']}")
        except Exception as e:
            print('[eqsmart] [consumer] [连接Provider] [ERROR]:' + str(e))
            print(traceback.format_exc())
            return json_dumps({'code': 'remote_call_error', 'message': str(e)})

        '''获取到本机IP'''
        send_data['ip'] = socket.gethostbyname(socket.gethostname())

        '''发送信息到Provider'''
        try:
            connect_provider.sendall(bytes(json_dumps(send_data), encoding="utf8"))
            print('[eqsmart] [consumer] [Provider服务调用] [调用信息]:', json_dumps(send_data))
            data = connect_provider.recv(self.server_conf['BUF_SIZE'])
            print('[eqsmart] [consumer] [Provider服务调用] [响应信息]:', str(data, 'UTF-8'))
            res = data
        except Exception as e:
            print('[eqsmart] [consumer] [Provider服务调用] [ERROR]:' + str(e))
            print(traceback.format_exc())
            return json_dumps({'code': 'remote_call_error', 'message': str(e)})
        """
        关闭注册连接
        """
        connect_provider.close()
        return res



"""
配置文件的读取
"""
from eqsmart.components.read_conf import ReadConf


class LoadConf:
    def __init__(self, path, start_params):
        """
        初始化
        :param path: 配置文件路径
        :param start_params: 应用启动参数
        """
        self.configuration_ext = {}
        self.configuration = ReadConf(path)
        self.configuration.__file_load__()
        if 'env' in start_params:
            ''' 启动参数传入运行环境信息时，额外加载环境配置文件 '''
            self.load_ext(path[:-5] + '-' + start_params['env'] + path[-5:])
        else:
            ''' 启动参数未传入运行环境信息，判断主配置文件是否指定环境信息 '''
            if 'env' in self.configuration.configuration['app']:
                self.load_ext(path[:-5] + '-' + self.configuration.configuration['app']['env'] + path[-5:])
        if 'port' in start_params:
            ''' 支持启动参数强制指定端口 '''
            self.configuration.configuration['app']['link_server']['port'] = int(start_params['port'])

    def node_read(self, conf_node):
        """
        读取服务启动配置信息
        :return: 配置信息节点
        """
        res = None
        try:
            res = self.configuration.__read__(conf_node)
        except Exception as e:
            print('[eqsmart] [read conf]', conf_node, 'not exist', str(e))
        return res

    def load_ext(self, path):
        """
        加载附加环境配置文件信息
        :param path: 附加环境路径
        :return: void
        """
        self.configuration_ext = ReadConf(path)
        self.configuration_ext.__file_load__()
        self.configuration.configuration.update(self.configuration_ext.configuration)


def get_run_params(input_params):
    """
    启动参数处理
    :param input_params: 应用启动参数 ['env=dev', 'port=7877']
    :return: 启动参数格式化 {'env':'dev', 'port':'7877'}
    """
    run_params = {}
    for item in input_params:
        try:
            info = item.split('=')
            run_params[info[0]] = info[1]
        except Exception as e:
            print('仅解析 a=b 格式的启动参数，其他跳过，', e)
    return run_params



import json
import socket
import sys
from threading import Thread
from eqsmart.components.protocol import *


def not_c_key(p_dict, p_key):
    """
    判断不包含元素
    :param p_dict: 字典
    :param p_key: key
    :return: p_dict 不包含 p_key, 返回 True
    """
    try:
        if p_dict.__contains__(p_key) is True and p_dict[p_key] is not None and p_dict[p_key] != '':
            return False
    except Exception as e:
        print('[eqsmart]', 'conf check', str(e))
    return True


class Provider:
    def __init__(self, provider_conf, server_info, server_list):
        self.provider_conf = {}
        self.server_info = server_info
        self.server_list = server_list
        self.provider_conf['HOST'] = '127.0.0.1' if not_c_key(provider_conf, 'HOST') else provider_conf['HOST']
        self.provider_conf['PORT'] = 7901 if not_c_key(provider_conf, 'PORT') else provider_conf['PORT']
        self.provider_conf['BACKLOG'] = 5 if not_c_key(provider_conf, 'BACKLOG') else provider_conf['BACKLOG']
        self.provider_conf['BUF_SIZE'] = 1024 if not_c_key(provider_conf, 'BUF_SIZE') else provider_conf['BUF_SIZE']
        self.provider_conf['IP'] = '127.0.0.1' if not_c_key(provider_conf, 'IP') else provider_conf['IP']
        self.provider_conf['WEIGHT'] = 1 if not_c_key(provider_conf, 'IP') else provider_conf['WEIGHT']

    def __send_data__(self):
        register_data = []
        send_data = {
            'type': 'provider register',
            'remote': {
                'ip': self.provider_conf['IP'],
                'port': self.provider_conf['PORT'],
                'weight': self.provider_conf['WEIGHT']
            },
            'service_name': '',
            'func': []
        }
        for item in self.server_info:
            send_data['service_name'] = item
            send_data['func'] = self.server_info[item]
            register_data.append(send_data.copy())
        return register_data

    def __func_call__(self, connect):
        while True:
            '''处理客户端端数据'''
            try:
                """
                recv(buffer_size) 接收TCP数据，数据以字符串形式返回，buffer_size 指定要接收的最大数据量
                """
                data = connect.recv(self.provider_conf['BUF_SIZE'])
                data = str(data, 'UTF-8')
                if data == '':
                    break
                data_json = json.loads(data)
                print('[eqsmart] [provider] 接收到客户端数据:', data_json)
                response = protocol_analysis(data_json)
                try:
                    call_func = self.server_list.copy()
                    for item in data_json['service_name']:
                        call_func = call_func[item]
                    method = getattr(call_func['func'], data_json['func'])
                    response = method(*tuple(data_json['args']), **data_json['kwargs'])
                except Exception as e:
                    print(e)
                print('[eqsmart] [provider] 返回给客户端数据:', response)
                connect.sendall(bytes(json.dumps(response).encode('utf-8')))
            except socket.error as e:
                print(str(e))
                break
        '''关闭客户端连接'''
        connect.close()

    def server_init(self, register):
        """
        服务端 socket 初始化
        :return: None
        """

        try:
            """
            创建套接字：socket.socket([family[, type[, proto]]])
            family: 套接字家族可以使 AF_UNIX 或者 AF_INET
            type: 套接字类型可以根据是面向连接的还是非连接分为 SOCK_STREAM 或 SOCK_DGRAM
            protocol: 一般不填默认为 0
            """
            server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        except Exception as e:
            print('[eqsmart] [Provider] [初始化异常] ' + str(e))
            sys.exit()

        try:
            """
            bind() 绑定地址(host,port)到套接字， 在AF_INET下，以元组(host,port)的形式表示地址
            """
            server.bind((self.provider_conf['HOST'], self.provider_conf['PORT']))
            print('[eqsmart] [Provider Starting] ' + self.provider_conf['HOST'] + ':' + str(self.provider_conf['PORT']))
        except socket.error as e:
            print("[eqsmart] Bind failed!" + str(e))
            sys.exit()
        print("[eqsmart] Provider bind complete")

        ''' 启动一个线程，将自身注册到注册中心 '''
        send_data = self.__send_data__()
        for i in send_data:
            Thread(target=register, args=(i,)).start()
        """
        listen(backlog) 开始TCP监听。backlog指定在拒绝连接之前，操作系统可以挂起的最大连接数量。该值至少为1，大部分应用程序设为5即可
        """
        server.listen(self.provider_conf['BACKLOG'])
        print("[eqsmart] Provider now listening")

        while True:
            """
            accept() 被动接受TCP客户端连接,(阻塞式)等待连接的到来
            """
            connect, addr = server.accept()
            print("[eqsmart] Provider connected with %s:%s " % (addr[0], str(addr[1])))
            """
            启动线程
            """
            Thread(target=self.__func_call__, args=(connect,)).start()



"""
公共工具
"""


def protocol_analysis(protocol):
    """
    对协议类型进行分析和处理
    :param protocol: 协议 JSON 内容
    :return: void
    """
    if protocol['type'] == 'provider response':
        print('协议类型', 'provider response')
        return {'code': '1000', 'type': 'provider response'}
    elif protocol['type'] == 'call provider':
        print('协议类型', 'call provider')
        return {'code': '1000', 'type': 'call provider'}



"""
扫描服务列表
"""
import os
import sys
import importlib


def scan_dir(path):
    """
    扫描服务列表路径下服务文件
    :param path: 待扫描路径（排除 . _ 开头的文件）
    :return: 服务列表
    """
    file_list = {'func': []}
    files = os.listdir(path)
    for file in files:
        if os.path.isdir(path + '/' + file):
            if file[0] != '.' and file[0] != '_':
                files_son = scan_dir(path + '/' + file)
                file_list[file] = files_son
        else:
            if file[0] != '.' and file[0] != '_':
                if file[-3:] == '.py' and file[0] != '_':
                    file_list['func'].append(file[:-3])
    return file_list


def load_services_class(services, son_module=''):
    """
    读取服务对象
    :return: 服务对象列表
    """
    services_class = {}
    for service in services:
        if not hasattr(services_class, service):
            services_class[service] = {}
        for op in services[service]:
            if op == 'func':
                common_service = CommonService()
                for func in services[service][op]:
                    module_name = (service + '.' + func) if son_module == '' else (
                            son_module + '.' + service + '.' + func)
                    func_name = importlib.import_module(module_name)
                    setattr(common_service, func, func_name.service)
                    if not hasattr(services_class[service], op):
                        services_class[service]['func'] = {}
                    services_class[service]['func'] = common_service
            else:
                son_module_name = service if son_module == '' else (son_module + '.' + service)
                services_class[service][op] = load_services_class({op: services[service][op]}, son_module_name)[op]
    return services_class


class CommonService(object):
    """
    公共对象，用于加载服务方法
    """
    pass


class ScanServices:
    def __init__(self, path):
        sys.path.append(path)
        self.scan_path = path
        self.services = {}

    def get_services_new(self):
        """
        服务名扫描
        :return: 服务名列表
        """
        services = scan_dir(self.scan_path)
        del services['func']
        self.services = services
        return services

    def get_services_class_new(self):
        """
        服务对象加载
        :return:
        """
        services_class = load_services_class(self.services)
        return services_class

    def get_services(self):
        """
        读取服务列表
        :return: 服务、方法的字典
        """
        return self.get_services_new()

    def get_services_class(self):
        """
        读取服务对象
        :return: 服务对象列表
        """
        return self.get_services_class_new()


def main():
    res = ScanServices('../').get_services_new()
    print(res)


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(
    name="eqsmart",
    version="0.0.10",
    keywords=("eqsmart", "微服务", "eqlink"),
    description="微服务框架",
    long_description="provider 和 http server 支持配置 host",
    license="GPL-2.0",

    url="https://gitee.com/jingenqiang/eqsmart.git",
    author="eq",
    author_email="eq_enqiang@163.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=['eqlink', 'PyYAML==6.0']
)

"""
项目打包
python setup.py bdist_egg     # 生成类似 eqsmart-0.0.1-py2.7.egg，支持 easy_install 
# 使用此方式
python setup.py sdist         # 生成类似 eqsmart-0.0.1.tar.gz，支持 pip
# twine 需要安装
twine upload dist/eqsmart-0.0.10.tar.gz
"""



import math
import random

from eqsmart.main.consumer import Consumer
from eqlink.components.remote_server import remote_server
from eqlink.main.clent import fail_server_list
import json

protocol = {
    'type': 'call provider',
    'service_name': '',
    'func': '',
    'args': (),
    'kwargs': {}
}


class RemoteCall:
    def __init__(self, service_path):
        self.service_path = service_path

    def func_call(self, params):
        if type(params) is tuple:
            protocol['args'] = params
        else:
            protocol['kwargs'] = params
        service_path = self.service_path.split('/')
        protocol['service_name'] = service_path[:-1]
        protocol['func'] = service_path[-1]
        ''' 远程服务列表 '''
        provider_service_list = remote_server.__get__()
        ''' 远程服务地址信息'''
        provider_server = provider_service_list[service_path[0]]['remote']
        ''' 远程服务加权调用：计算权重 '''
        weight_check = {'0': 1}
        count_w = 0  # 用于远程服务遍历的计数器
        count_c = 0  # 权值和
        for item in provider_server:
            weight_check[str(count_w)] = item['weight']
            count_w = count_w + 1
            count_c = int(item['weight']) + count_c
        weight_factor = math.ceil(100 / count_c)  # 权重因子
        weight_m = 0  # 中间值
        call_random = random.randint(0, 100)  # 随机数
        call_weight = 0  # 加权计算后，被调用的远程服务
        print('[eqsmart] 远程服务加权调用，服务权重:', weight_check)
        for item in weight_check.keys():
            c_weight = int(weight_check[item]) * weight_factor
            weight_check[item] = [weight_m, weight_m + c_weight]
            if weight_m <= call_random <= weight_m + c_weight:
                call_weight = int(item)
            weight_m = weight_m + c_weight
        print('[eqsmart] [加权调用]', call_weight, call_random)
        ''' 以上为加权调用的权重计算 '''
        provider_conf = {
            'IP': provider_server[call_weight]['ip'],  # 远程服务器地址
            'PORT': provider_server[call_weight]['port'],  # 远程服务器端口
            'BUF_SIZE': 4096  # 消息读取长度
        }
        print('[eqsmart] 远程调用', provider_conf, protocol)
        try:
            remote_call = Consumer(provider_conf).func_call_int(protocol)
        except Exception as e:
            print('[eqsmart] 远程调用失败！', e)
            remote_call = json.dumps({'code': 'remote_call_error', 'message': str(e)})
        ''' 调用结果判断 '''
        res = json.loads(remote_call)
        try:
            if res['code'] == 'remote_call_error':
                fail_server_list.append({'IP': provider_conf['IP'], 'PORT': provider_conf['PORT']})
        except Exception as e:
            ''' 远程调用可能不返回 code '''
            print('远程调用成功！', e)
        return res



"""
子模块读取
"""

from importlib import import_module
from sys import path as sys_path


class ReadModule:
    def __init__(self, path=None):
        if path is None:
            path = '..'
        sys_path.append(path)
        self.module_list = {}

    def read(self, item):
        if item in self.module_list.keys():
            return self.module_list[item]
        module_item = import_module(item)
        self.module_list[item] = module_item
        return module_item
